#!/bin/sh
# ----------------------------------------------------------------------------
# /var/install/config.d/teamspeak3 - TEAMSPEAK3 configuration
#
# Creation   : 2010-01-26 christian
# Last update: $Id: teamspeak3.sh 37 2010-08-04 11:08:54Z root $
#
# Copyright (c) 2010 Christian Richter, <richterch(at)gmx(dot)de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> `pwd`/teamspeak3-trace$$.log
#set -x

# ----------------------------------------------------------------------------
# Config Dateien
# ----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

# include config
config_file=/etc/config.d/teamspeak3
. ${config_file}

# set package name
packageName=teamspeak3

# set some variables
temp='/tmp/teamspeak3.conf'
packagepath="/usr/local/${packageName}/"
server_ini="${packagepath}ts3server.ini"
db_ini="${packagepath}ts3db_mysql.ini"
#backup_path="${packagepath}bak/"
backup_path="/home/.${packageName}/"
mysqlfile="ts3server.mysql"
sqlitefile="ts3server-sqlitedb"
mysql_basedir='/usr/local/mysql'
generate_logrotate=/etc/logrotate.d/${packageName}
packageVersion=`grep "<version>" /var/install/packages/${packageName} | sed "s#<version>##g" | sed "s#</version>##g"`

# ----------------------------------------------------------------------------
# functions
# ----------------------------------------------------------------------------

mysqld_get_param()
{
        ${mysql_basedir}/bin/mysqld --print-defaults \
                | tr " " "\n" \
                | grep -- "--$1" \
                | tail -n 1 \
                | cut -d= -f2
}

createServerINI() {
    case ${DATABASE} in
        sqlite)
            dbplugin='ts3db_sqlite3'
            dbcreatepath='create_sqlite/'
        ;;
        mysql)
            dbplugin='ts3db_mysql'
            dbcreatepath='create_mysql/'
            createDBini
        ;;
    esac
    rm -f ${server_ini}

    cat >> ${server_ini} <<EOFG
machine_id=${TEAMSPEAK3_MACHINE_ID}
default_voice_port=${TEAMSPEAK3_VOICE_PORT}
voice_ip=${TEAMSPEAK3_VOICE_IP}
filetransfer_ip=${TEAMSPEAK3_FILETRANSFER_IP}
query_ip=${TEAMSPEAK3_QUERY_IP}
filetransfer_port=${TEAMSPEAK3_FILETRANSFER_PORT}
query_port=${TEAMSPEAK3_QUERY_PORT}
licensepath=${TEAMSPEAK3_LICENSEPATH}/
dbplugin=${dbplugin}
dbpluginparameter=
dbsqlpath=sql/
dbsqlcreatepath=${dbcreatepath}
logpath=logs
logquerycommands=0

EOFG
    chmod 0644 ${server_ini}
    chown teamspeak3:tss3 ${server_ini}
}

createDBini() {
    rm -f ${db_ini}
    cat >> ${db_ini} <<EOFG
[config]
host=${TEAMSPEAK3_MYSQL_HOST}
port=${TEAMSPEAK3_MYSQL_PORT}
username=${TEAMSPEAK3_MYSQL_USER}
password=${TEAMSPEAK3_MYSQL_PASS}
database=${TEAMSPEAK3_MYSQL_DB}
socket=${TEAMSPEAK3_MYSQL_SOCKET}

EOFG
    chmod 0644 ${db_ini}
    chown teamspeak3:tss3 ${db_ini}

}


checkDatabase() {
    case "${TEAMSPEAK3_USE_MYSQL}" in
        yes)
            /var/install/bin/check-package -p mysql "TEAMSPEAK3_USE_MYSQL='yes'"
            retvar="${?}"
            if [ "${retvar}" -eq 0 ]
            then
                DATABASE=mysql
                mysql_pidfile=`mysqld_get_param pid-file`
                mysql_datadir=`mysqld_get_param datadir`
                check_mysql_network
            else
                mecho -warn "Package mysql not installed, so we use SQLite"
                echo
                # set parameter from 'yes' to 'no'
                sed "s/^TEAMSPEAK3_USE_MYSQL=.*/TEAMSPEAK3_USE_MYSQL='no'/" ${config_file} >/tmp/CONFIG_FILE_TMP
                mv /tmp/CONFIG_FILE_TMP ${config_file}
                chmod 0600 ${config_file}
                TEAMSPEAK3_USE_MYSQL='no'
                DATABASE=sqlite
           fi

            ;;
        no)
            DATABASE=sqlite
            ;;
    esac
}

MySQL_helper() {
    if [ -f "${mysql_pidfile}" ]
    then
        if [ -f ${mysql_datadir}/backup.pwd ]
        then
            passwd=`grep passwd= ${mysql_datadir}/backup.pwd | sed 's/passwd=//'`
            helper='true'
        fi
    fi
}
importDATABASE() {
    /etc/init.d/teamspeak3 stop > /dev/null
    dropDATABASEandUSER
    createDATABASEandUSER
    mysql --user=${TEAMSPEAK3_MYSQL_USER} --password=${TEAMSPEAK3_MYSQL_PASS} ${TEAMSPEAK3_MYSQL_DB} < ${backup_path}${mysqlfile}
}

dropDATABASEandUSER() {
    mysql -u backup -p${passwd} -e "drop database if exists ${TEAMSPEAK3_MYSQL_DB};"
    mysql -u backup -p${passwd} -e "USE mysql; DELETE FROM user WHERE User = '${TEAMSPEAK3_MYSQL_USER}'; DELETE FROM db WHERE User = '${TEAMSPEAK3_MYSQL_USER}'; DELETE FROM tables_priv WHERE User = '${TEAMSPEAK3_MYSQL_USER}'; DELETE FROM columns_priv WHERE User = '${TEAMSPEAK3_MYSQL_USER}';"
}

createDATABASEandUSER() {
    mysql -u backup -p${passwd} -e "GRANT  SELECT , INSERT , UPDATE , DELETE , CREATE , DROP , INDEX , ALTER , LOCK TABLES ON ${TEAMSPEAK3_MYSQL_DB} . * TO '${TEAMSPEAK3_MYSQL_USER}'@'${HOSTNAME}'IDENTIFIED BY '${TEAMSPEAK3_MYSQL_PASS}' ; GRANT  SELECT , INSERT , UPDATE , DELETE , CREATE , DROP , INDEX , ALTER , LOCK TABLES ON ${TEAMSPEAK3_MYSQL_DB} . * TO '${TEAMSPEAK3_MYSQL_USER}'@'localhost'IDENTIFIED BY '${TEAMSPEAK3_MYSQL_PASS}' ; "
    mysql -u backup -p${passwd} -e "CREATE DATABASE IF NOT EXISTS ${TEAMSPEAK3_MYSQL_DB};"

}

check_mysql_network()
{
    if [ -f "/etc/config.d/mysql" ]
    then
        mecho ""
        mecho -n "Checking if 'MYSQL_NETWORK' on mysql configuration activated... "
        . /etc/config.d/mysql
        value=$MYSQL_NETWORK
        case ${value} in
            'yes'|'YES'|'Yes')
                mecho "Done"
                ;;
            *)
                mecho ""
                mecho -warn "Option 'MYSQL_NETWORK' is set to '${value}'!"
                if /var/install/bin/ask "Do you want to edit the mysql configuration now" "yes"
                then
                    sh /var/install/bin/edit -apply mysql "Edit mysql configuration"
                else
                    mecho ""
                    mecho -warn "You won't be able to start the teamspeak3 server. "
                    exit 1
                fi
                ;;
        esac
    else
        mecho -warn "Could not check the setting 'MYSQL_NETWORK' ('/etc/config.d/mysql' not found). You"
        mecho -warn "have to check this by hand!"
    fi
}

#-----------------------------------------------------------------------------
# create logrotate file
#-----------------------------------------------------------------------------
create_logrotate ()
{
    mecho -n "Creating logrotate configuration file..."

    {
        #-------------------------------------------------------------------------------
        print_short_header "${generate_logrotate}" "${packageName}" "${packageVersion}"
        #-------------------------------------------------------------------------------
        echo
        if [ "{START_TEAMSPEAK3}" == "yes" ]
        then
            echo "${TEAMSPEAK3_LOG_PATH}/ts3.log {"
            echo "    rotate ${TEAMSPEAK3_LOGROTATE_COUNT}"
            echo "    ${TEAMSPEAK3_LOGROTATE_INTERVAL}"
            echo "    compress"
            echo "    notifempty"
            echo "    postrotate"
            echo "       sh /etc/init.d/${packageName} restart"                       
            echo "    endscript"
            echo "    }"
        else
            echo "# ${packageName} disabled!"
        fi
    } >${generate_logrotate}
    mecho " Done."
}

Check_Sendmail() {
    if [ ! -f /var/install/packages/mail ]
    then
        if [ ! -f /var/install/packages/vmail ]
        then
            if [ ! -f /var/install/packages/ssmtp ]
            then
                mecho -error "There is no mail package found. Please check this first."
                mecho -error "disable TEAMSPEAK3_UPDATE_CHECK."
                sed -i "/^TEAMSPEAK3_UPDATE_CHECK=/s/=.*/='no'/" /etc/config.d/teamspeak3
            fi
        fi
    fi

}





# ----------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------
rm -f ${packagepath}ts3.log
checkDatabase
case "${DATABASE}" in
    mysql)
        if [ -f "${packagepath}${sqlitefile}" ]
        then
            mkdir -p ${backup_path}
            mv -f ${packagepath}${sqlitefile} ${backup_path}${sqlitefile}
        fi
        if [ -f "${packagepath}pass.log" ]
        then
            mkdir -p ${backup_path}
            mv -f ${packagepath}pass.log ${backup_path}pass.log.sqlite-bak
        fi
        MySQL_helper
        if [ "${helper}" == 'true' ]
        then
            if [ -f "${backup_path}${mysqlfile}" ]
            then
                mecho -info "found a database file in ${backup_path}."
                if /var/install/bin/ask "do you want to use it?" "yes"
                then
                    importDATABASE
                    rm -f ${backup_path}${mysqlfile}
                    if [ -f "${backup_path}pass.log.mysql-bak" ]
                    then
                        mv -f ${backup_path}pass.log.mysql-bak ${packagepath}pass.log
                    fi
                else
                    createDATABASEandUSER
                fi
            else
                createDATABASEandUSER
            fi
        fi
        ;;
    sqlite)
        MySQL_helper
        if [ "${helper}" == 'true' ]
        then
            if DBS=`mysql -u backup -h localhost -p${passwd} -Bse 'show databases' | grep ${TEAMSPEAK3_MYSQL_DB}`
            then
                if [ "${TEAMSPEAK3_MYSQL_USER}" != "" ] && [ "${TEAMSPEAK3_MYSQL_PASS}" != "" ] && [ "${TEAMSPEAK3_MYSQL_DB}" != "" ]
                then
                    mkdir -p ${backup_path}
                    exportfile="${backup_path}${mysqlfile}"
                    mysqldump --user=${TEAMSPEAK3_MYSQL_USER} --password=${TEAMSPEAK3_MYSQL_PASS} --add-drop-table -a ${TEAMSPEAK3_MYSQL_DB} > ${exportfile}
                fi
                dropDATABASEandUSER
            fi
        fi
        if [ -f "${packagepath}pass.log" ]
        then
            mkdir -p ${backup_path}
            mv -f ${packagepath}pass.log ${backup_path}pass.log.mysql-bak
        fi
        if [ -f "${backup_path}${sqlitefile}" ]
        then
            mecho -info "found a database file in ${backup_path}."
            if /var/install/bin/ask "do you want to use it?" "yes"
            then
                mv -f ${backup_path}${sqlitefile} ${packagepath}${sqlitefile}
                if [ -f "${backup_path}pass.log.sqlite-bak" ]
                then
                    mv -f ${backup_path}pass.log.sqlite-bak ${packagepath}pass.log
                fi
            fi
        fi
        ;;
esac
rm -f ${packagepath}*.ini
createServerINI
if [ "${TEAMSPEAK3_CLEAR_DB}" == 'yes' ]
then
    _ts3_config_tmpfile=`mktemp -t ts3_config_tmpfile.XXXXXXXXXX`
    sed "s/^TEAMSPEAK3_CLEAR_DB=.*/TEAMSPEAK3_CLEAR_DB='no'/" ${config_file} >${_ts3_config_tmpfile}
    mv ${_ts3_config_tmpfile} ${config_file}
    chmod 0600 ${config_file}
fi
chown -R teamspeak3:tss3 ${packagepath}
if [ "${TEAMSPEAK3_UPDATE_CHECK}" == 'yes' ]
then
    Check_Sendmail
fi

exit 0
# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------
